<?php

namespace Drupal\datetime_more\Element;

use Drupal\Core\Datetime\Element\Datelist;
use Drupal\Core\Form\FormStateInterface;

/**
 * Provides a datelist_more element.
 *
 * @FormElement("datelist_more")
 */
class DatelistMore extends Datelist {

  /**
   * Expands a date element into an array of individual elements.
   *
   * Required settings:
   *   - #default_value: A DrupalDateTime object, adjusted to the proper local
   *     timezone. Converting a date stored in the database from UTC to the local
   *     zone and converting it back to UTC before storing it is not handled here.
   *     This element accepts a date as the default value, and then converts the
   *     user input strings back into a new date object on submission. No timezone
   *     adjustment is performed.
   * Optional properties include:
   *   - #date_part_order: Array of date parts indicating the parts and order
   *     that should be used in the selector, optionally including 'ampm' for
   *     12 hour time. Default is array('year', 'month', 'day', 'hour', 'minute').
   *   - #date_text_parts: Array of date parts that should be presented as
   *     text fields instead of drop-down selectors. Default is an empty array.
   *   - #date_date_callbacks: Array of optional callbacks for the date element.
   *   - #date_year_range: A description of the range of years to allow, like
   *     '1900:2050', '-3:+3' or '2000:+3', where the first value describes the
   *     earliest year and the second the latest year in the range. A year
   *     in either position means that specific year. A +/- value describes a
   *     dynamic value that is that many years earlier or later than the current
   *     year at the time the form is displayed. Defaults to '1900:2050'.
   *   - #date_increment: The increment to use for minutes and seconds, i.e.
   *     '15' would show only :00, :15, :30 and :45. Defaults to 1 to show every
   *     minute.
   *   - #date_timezone: The Time Zone Identifier (TZID) to use when displaying
   *     or interpreting dates, i.e: 'Asia/Kolkata'. Defaults to the value
   *     returned by date_default_timezone_get().
   *
   * Example usage:
   * @code
   *   $form = array(
   *     '#type' => 'datelist_more',
   *     '#default_value' => new DrupalDateTime('2000-01-01 00:00:00'),
   *     '#date_part_order' => array('month', 'day', 'year', 'hour', 'minute', 'ampm'),
   *     '#date_text_parts' => array('year'),
   *     '#date_year_range' => '2010:2020',
   *     '#date_increment' => 15,
   *     '#date_timezone' => 'Asia/Kolkata'
   *   );
   * @endcode
   *
   * @param array $element
   *   The form element whose value is being processed.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   * @param array $complete_form
   *   The complete form structure.
   *
   * @return array
   */
  public static function processDatelist(&$element, FormStateInterface $form_state, &$complete_form) {

    $text_parts = !empty($element['#date_text_parts']) ? $element['#date_text_parts'] : [];
//    $year_min = !empty($element['#year_min']) ? $element['#year_min'] : 1;
//    $year_max = !empty($element['#year_max']) ? $element['#year_max'] : 2999;

    $element = parent::processDatelist($element, $form_state, $complete_form);

    $part = 'year';
    if (in_array('year', $text_parts)) {
      $element['year']['#type'] = 'number';
      $element['year']['#size'] = 1;
      $element['year']['#maxlength'] = 4;
      $range = static::datetimeRangeYears($element['#date_year_range']);
      $element['year']['#min'] = $range[0];
      $element['year']['#max'] = $range[1];
    }
    if (in_array('month', $text_parts)) {
      $element['month']['#type'] = 'number';
      $element['month']['#size'] = 1;
      $element['month']['#maxlength'] = 2;
      $element['month']['#min'] = 1;
      $element['month']['#max'] = 12;
    }
    if (in_array('day', $text_parts)) {
      $element['day']['#type'] = 'number';
      $element['day']['#size'] = 1;
      $element['day']['#maxlength'] = 2;
      $element['day']['#min'] = 1;
      $element['day']['#max'] = 31;
    }
    if (in_array('hour', $text_parts)) {
      $element['hour']['#type'] = 'number';
      $element['hour']['#size'] = 1;
      $element['hour']['#maxlength'] = 2;
      $element['hour']['#min'] = 0;
//      $element['hour']['#max'] = $element['#time_type'];
      $element['hour']['#max'] = 23;
    }
    if (in_array('minute', $text_parts)) {
      $element['minute']['#type'] = 'number';
      $element['minute']['#size'] = 1;
      $element['minute']['#maxlength'] = 2;
      $element['minute']['#min'] = 0;
      $element['minute']['#max'] = 59;
    }
    if (in_array('second', $text_parts)) {
      $element['second']['#type'] = 'number';
      $element['second']['#size'] = 1;
      $element['second']['#maxlength'] = 2;
      $element['second']['#min'] = 0;
      $element['second']['#max'] = 59;
    }

    return $element;
  }

}
